using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;

namespace Parallax.Serial.IO
{
	//=========================================================================
	/// <summary>
	/// Represents the parallax basic stamp based hardware. Provides basic 
	/// connection and communication services to the basic stamp via serial ports.
	/// Inherit from this class to add more functionality.
	/// </summary>
	public abstract class RobotBase
	{
		//=========================================================================
		#region -= declarations =-

		//---- internal bits
		protected System.IO.Ports.SerialPort m_prtSerial = new SerialPort();
		protected bool m_bolInitialized;
		protected string m_strSerialPortName;

		//---- delegates
		public delegate void RobotDataReceivedEventHandler(object sender, SerialDataReceivedEventArgs e);
		//---- events
		public event RobotDataReceivedEventHandler DataReceived;


		#endregion
		//=========================================================================

		//=========================================================================
		#region -= properties =-

		/// <summary>
		/// Provides access to the underlying port. eventually i want to encapsulate 
		/// this better and insulate consumers from it, but for now it's here.
		/// </summary>
		public SerialPort Port
		{
			get { return this.m_prtSerial; }
		}
		/// <summary>
		/// The name of the serial port to connect to the stamp via. Defaults to "COM1"
		/// </summary>
		/// <remarks>Note, changing the serial port will result in any existing 
		/// port connection to be closed.</remarks>
		public string SerialPortName
		{
			get { return this.m_strSerialPortName; }
			set
			{
				//---- declare vars
				bool isDifferent = (this.m_strSerialPortName != value && this.m_strSerialPortName != "");

				//---- if the port has changed, and the port is open, close the first one
				if (isDifferent && this.PortIsOpen)
				{ this.Close(); }

				//---- set the value
				this.m_strSerialPortName = value;

				//---- if the port has changed, re-initialize
				if (isDifferent)
				{
					this.m_bolInitialized = false;
					this.Initialize();
				}
			}
		}
		/// <summary>
		/// Whether or not the port is opened.
		/// </summary>
		public bool PortIsOpen
		{
			get { return this.m_prtSerial.IsOpen; }
		}

		#endregion
		//=========================================================================

		//=========================================================================
		#region -= constructors =-

		/// <summary>
		/// Instantiates a new RobotBase class. Sets SerialPortName to "COM1" by default.
		/// </summary>
		public RobotBase()
		{
			this.CommonInit();
		}

		protected void CommonInit()
		{
			this.m_strSerialPortName = "COM1";
		}

		#endregion
		//=========================================================================

		//=========================================================================
		#region -= private methods =-

		//=========================================================================
		/// <summary>
		/// Initializes a serial connection via the serial port to the robot. 
		/// (9600, 8, 1, N)
		/// </summary>
		/// <remarks>
		/// Does not open a connection. It simply configures it.
		/// </remarks>
		private void Initialize()
		{
			//---- set the com port we are using from the PC
			m_prtSerial.PortName = this.m_strSerialPortName;

			//---- set the baud rate (the rate that data is sent and received in bits/sec)
			m_prtSerial.BaudRate = 9600;
			
			//---- set number of bits in each frame of data
			m_prtSerial.DataBits = 8;

			//---- parity bit for error detection purposes: even, odd, or none
			m_prtSerial.Parity = Parity.None;

			//---- set stop bit to one (used to separate the data into frames)
			m_prtSerial.StopBits = StopBits.One;

			//---- no handshaking used
			m_prtSerial.Handshake = Handshake.None;

			//---- wire up the data received event
			m_prtSerial.DataReceived += new SerialDataReceivedEventHandler(m_prtSerial_DataReceived);

			//---- set initialized to true, so that it doesn't get initialized twice
			this.m_bolInitialized = true;
		}
		//=========================================================================

		#endregion
		//=========================================================================

		//=========================================================================
		#region -= public methods =-

		//=========================================================================
		/// <summary>
		/// Opens a connection to the port specified in <see cref="SerialPortName"/>
		/// (if it is not open) and sends the <paramref name="instructionString"/> to 
		/// the device.
		/// </summary>
		/// <remarks>
		/// Appends a newline character to the end of the string which tells the chip 
		/// that the string has terminated.
		/// </remarks>
		/// <param name="instructionString"></param>
		public void SendInstructions(string instructionString)
		{
			//---- if the port isn't open yet, open it.
			if (!this.m_prtSerial.IsOpen)
			{ this.m_prtSerial.Open(); }
			
			//---- send the string over the serial port
			// note: writeline adds a newline character to the end which tells 
			// the chip when the string is terminated.
			m_prtSerial.WriteLine(instructionString);
		}
		//=========================================================================

		//=========================================================================
		/// <summary>
		/// Opens the underlying port connection to the robot. If you are waiting 
		/// for recieved data, you must keep the port open. However, you should 
		/// close the port when you're finished. 
		/// </summary>
		/// <param name="serialPortName">Name of the serial port, such as "COM1"</param>
		public void Open()
		{
			//---- if the port hasn't been initialized, initialize it
			if (!this.m_bolInitialized)
			{ this.Initialize(); }

			//---- if the underlying port isn't open, open it
			if (!this.m_prtSerial.IsOpen)
			{ this.m_prtSerial.Open(); }
		}
		//=========================================================================

		//=========================================================================
		/// <summary>
		/// Closes the underlying port connection to the robot.
		/// </summary>
		/// <remarks>
		/// If you are waiting for recieved data, you must keep the port open. 
		/// However, you should close the port when you're finished. 
		/// </remarks>
		public void Close()
		{
			//---- close the port if it's open
			if (this.m_prtSerial.IsOpen)
			{ this.m_prtSerial.Close(); }
		}
		//=========================================================================

		//=========================================================================
		/// <summary>
		/// Closes the underlying port connection
		/// </summary>
		public void Dispose()
		{
			//---- close the port
			this.Close();
		}
		//=========================================================================

		#endregion
		//=========================================================================

		//=========================================================================
		#region -= event handlers =-

		//=========================================================================
		/// <summary>
		/// DataReceived event handler.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		void m_prtSerial_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
			if (this.DataReceived != null)
			{
				this.DataReceived(this, e);
			}
		}
		//=========================================================================

		#endregion
		//=========================================================================

	}
	//=========================================================================
}
